// Fragment Shader  file "shadername.frag"

#version 330

out vec4 vFragColor;

uniform vec4		diffuseColor;
uniform sampler2D	colorTexture;
uniform sampler2D	alphaTexture;
uniform sampler2D	projTexture;	
uniform int			useLight;
uniform int			useTexture;
uniform float		materialFactor;
uniform vec3		vLightPosition;
uniform float		lightFactor;
//uniform vec3		vRandom;
uniform float		ambientFactor;

in vec2 vVaryingTexCoords;
in vec3 vVaryingNormal;
in vec3 vVaryingPosition;
in vec4 vVaryingTexCoordsProj;
//in vec3 vVaryingRandom;
void main(void)
{
	//projective texture hints:
	//fragment shader
		//in vec4 SM_tex_coord; //fragment shader in_varying
		//normal approach:
			// perform perspective division
			//vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;
			// read depth value from shadow map
			//float depth = texture(shadow_map, tex_coords.xy).r;
		//hardware assisted approach: 
			//textureProj(shadow_map, SM_tex_coord);
	vec3 vVaryingLightDir = normalize(vLightPosition - vVaryingPosition);
	vec4 texColor, texColorProj;
	//vec3 lightningPos;
	//lightningPos.x = lightningPos.y = lightningPos.z = 10;
	if (useTexture == 2 && texture(colorTexture, vVaryingTexCoords.st).a <= 0.3f) { discard; } //alpha channel for leaves 
	if (useTexture == 3 && texture(colorTexture, vVaryingTexCoords.st).g <= 0.15f) { discard; } //green channel alpha for lightning

	if (useTexture == 0) //no textures
	{
		texColor = diffuseColor;
	}
	else 
	{	
		texColor = texture(colorTexture, vVaryingTexCoords.st);
		if (vVaryingTexCoordsProj.w > 0)
		{
			texColorProj = textureProj(projTexture, vVaryingTexCoordsProj);		
			texColor += texColorProj;
		}
		if (useTexture == 3) //alphamap
		{
			texColor.a = (texture(alphaTexture, vVaryingTexCoords.st).x);
		}
		else if (useTexture == 4) //combine texture with diffusecolor
		{
			texColor = texColor + diffuseColor;
		}
		else if (useTexture == 5) //random colors added
		{
			//texColor.rgb = texColor.rgb * vRandom; 
			//texColor.rgb = vVaryingRandom;
			texColor.a = 1;
		}
	}

	if (useLight == 0)
	{
		vFragColor = texColor;
	}
	else
	{
		vec4 ambientTexColor = texColor * ambientFactor;
		// Add in ambient light
		vFragColor = ambientTexColor;

		if (lightFactor >= 0.01)
		{
			// Dot product gives us diffuse intensity
			float diff = max(0.0, dot(normalize(vVaryingNormal), normalize(vVaryingLightDir)));

			// Multiply intensity by diffuse color, force alpha to 1.0
			//vFragColor = diff * diffuseColor;
			vFragColor += diff * texColor;

			// Specular Light
			vec3 vReflection = normalize(reflect(-normalize(vVaryingLightDir), normalize(vVaryingNormal)));
			float spec = max(0.0, dot(normalize(-vVaryingPosition), vReflection));

			// If the diffuse light is zero, don't even bother with the pow function

			if(diff != 0 && materialFactor != 0) {
				float fSpec = pow(spec, 128.0);
				vFragColor.rgb += vec3(fSpec, fSpec, fSpec) * materialFactor;
			}
		}
		//setting alpha value independent from lightcalc
		vFragColor.a = texColor.a;
	}
	//if (useLight != 0) {vFragColor.rgb *= lightFactor * 2; }
	//vFragColor.rgb = vVaryingNormal;
	//vFragColor.r = vVaryingTexCoordsProj.x;
	//vFragColor.g=vVaryingTexCoordsProj.z;
	//vFragColor.b=0;
	//vFragColor.a=1;
}
