
#version 330

// Incoming per vertex... position and normal
in vec4 vVertex;
in vec3 vNormal;
in vec2 vTexCoord;

// Set per batch
uniform vec4    ambientColor;
uniform vec4    diffuseColor;
uniform vec4    specularColor;

uniform vec3    vLightPosition;
uniform vec3	vEyePosition;
uniform mat4    mvpMatrix;
uniform mat4    mvMatrix;
uniform mat3    normalMatrix;
uniform int     useLight;

// Color to fragment program
smooth out vec4 vVaryingColor;


void main(void)
    {
    if (useLight == 1)
	{
		// Get surface normal in eye coordinates
		vec3 vEyeNormal = normalMatrix * vNormal;

		// Get vertex position in eye coordinates
		vec4 vPosition4 = mvMatrix * vVertex;
		vec3 vPosition3 = vPosition4.xyz / vPosition4.w;

		// Get vector to light source
		vec3 vLightDir = normalize(vLightPosition - vPosition3);



		// Dot product gives us diffuse intensity
		float diff = max(0.0, dot(vEyeNormal, vLightDir));

		// Multiply intensity by diffuse color, force alpha to 1.0
		vVaryingColor = diff * diffuseColor;
		//vVaryingColor = vec4(0,0,0,1);
		// Add in ambient light
		vVaryingColor += ambientColor;

		// Specular Light
		vec3 vReflection = normalize(reflect(-vLightDir, vEyeNormal));
		float spec = max(0.0, dot(normalize(-vPosition3), vReflection));
		if(diff != 0) {
			float fSpec = pow(spec, 128.0);
			vVaryingColor.rgb += vec3(fSpec, fSpec, fSpec);
		}	
	}

	else //Just apply the color
	{
		vVaryingColor = diffuseColor;
	}

    // Don't forget to transform the geometry!
    gl_Position = mvpMatrix * vVertex;
    }

	/*
	
#version 330

// Incoming per vertex... position and normal
in vec4 vVertex;
in vec3 vNormal;
in vec2 vTexCoord;

// Set per batch
uniform vec4    diffuseColor;
uniform vec3    vLightPosition;
uniform mat4    mvpMatrix;
uniform mat4    mvMatrix;
uniform mat3    normalMatrix;
uniform int     useLight;


// Color to fragment program
smooth out vec4 vVaryingColor;
 out vec4 vPos;

void main(void)
{
    if (useLight == 1)
	{
		vec3 vEyeNormal = normalMatrix * vNormal;

		// Get vertex position in eye coordinates
		vec4 vPosition4 = mvMatrix * vVertex;
		vec3 vPosition3 = vPosition4.xyz / vPosition4.w;

		// Get vector to light source
		vec3 vLightDir = normalize(vLightPosition - vPosition3);

		// Dot product gives us diffuse intensity
		float diff = max(0.0, dot(vEyeNormal, vLightDir));

		// Multiply intensity by diffuse color, force alpha to 1.0
		vVaryingColor.xyz = diff * diffuseColor.xyz;
		vVaryingColor.a = 1.0;
		vPos.xyz = vLightDir;
	}
	
	else //Just apply the color
	{
		vVaryingColor = diffuseColor;
	}
	
	// Let's not forget to transform the geometry
    gl_Position = mvpMatrix * vVertex;
    }
	*/
					  