// Fragment Shader  file "shadername.frag"

#version 330

out vec4 vFragColor;

uniform vec4		diffuseColor;
uniform sampler2D	colorTexture;
uniform sampler2D	alphaTexture;

uniform int			useTexture;
uniform int			bubbleCursor;
uniform int			showPickBufferRectangle;
uniform ivec2		mousePos;
uniform float		bubbleMaxDistance;
uniform ivec2		recLowerLeft;
uniform ivec2		recSize;


in vec2 vVaryingTexCoords;

void main(void)
{
	vec4 texColor = vec4(0.0f,0.0f,0.0f,0.0f);

	if (useTexture == 0) //no textures
	{
		texColor = diffuseColor;
	}
	else 
	{	
		vec2 pixelPos = gl_FragCoord.xy;
		if (bubbleCursor == 1)  //red transparent bubblecursor
		{ 
			float dist = distance(pixelPos,vec2(mousePos.x, mousePos.y));
			if (dist < bubbleMaxDistance) { texColor = vec4(1.0, 0.0, 0.0, 0.3); }
			else {discard; }
		}
		
		else if (showPickBufferRectangle == 1) //blue transparent pickbuffer visualization
		{
			//pixelPos is X from Left to Right and Y from Bottom to Top
			ivec2 recUpperRight = recLowerLeft + recSize;

			if (pixelPos.x >= recLowerLeft.x && pixelPos.x < recUpperRight.x && pixelPos.y >= recLowerLeft.y && pixelPos.y < recUpperRight.y ) { texColor = vec4(0.0, 0.0, 1.0, 0.3); }
			else {discard; }
		}

		else //crosshair texture cursor
		{
			texColor = texture(colorTexture, vVaryingTexCoords.st);
			if (texColor.r > 0.9 && texColor.g > 0.9 && texColor.b > 0.9) { texColor.a = 1 - texColor.r; }
		}

	}
	if (useTexture == 3) //alphamap
	{
		texColor.a = (texture(alphaTexture, vVaryingTexCoords.st).x);
	}

	vFragColor = texColor;
}
